@@ -143,6 +143,7 @@ static int release_inode_block(ext2_filsys fs,
 	}
 
 	ext2fs_block_alloc_stats2(fs, blk, -1);
+	ctx->free_blocks++;
 	return retval;
 }
 
@@ -211,9 +212,11 @@ static int release_inode_blocks(e2fsck_t ctx, ext2_ino_t ino,
 				ino);
 			return 1;
 		}
-		if (count == 0)
+		if (count == 0) {
 			ext2fs_block_alloc_stats2(fs,
 					ext2fs_file_acl_block(fs, inode), -1);
+			ctx->free_blocks++;
+		}
 		ext2fs_file_acl_block_set(fs, inode, 0);
 	}
 	return 0;
@@ -286,6 +289,7 @@ static int release_orphan_inodes(e2fsck_t ctx)
 		if (!inode.i_links_count) {
 			ext2fs_inode_alloc_stats2(fs, ino, -1,
 						  LINUX_S_ISDIR(inode.i_mode));
+			ctx->free_inodes++;
 			inode.i_dtime = ctx->now;
 		} else {
 			inode.i_dtime = 0;
@@ -635,6 +639,8 @@ void check_super_block(e2fsck_t ctx)
 
 		should_be = 0;
 		if (!ext2fs_group_desc_csum_verify(fs, i)) {
+			pctx.csum1 = ext2fs_bg_checksum(fs, i);
+			pctx.csum2 = ext2fs_group_desc_csum(fs, i);
 			if (fix_problem(ctx, PR_0_GDT_CSUM, &pctx)) {
 				ext2fs_bg_flags_clear(fs, i, EXT2_BG_BLOCK_UNINIT);
 				ext2fs_bg_flags_clear(fs, i, EXT2_BG_INODE_UNINIT);
